#ifndef _OctreeHierarchyInterface_H__
#define _OctreeHierarchyInterface_H__

#include <OgreOctree.h>
#include <OgreOctreeSceneManager.h>
#include "OgreSceneNodeHierarchyInterface.h"

namespace Ogre {
/**
	This class implements the hierarchy interface for the Ogre octree hierarchy.
*/
class OctreeHierarchyInterface : public SceneNodeHierarchyInterface
{
public:
	OctreeHierarchyInterface(OctreeSceneManager *sm, RenderSystem *rsys);
	
	/** Sets the number of nodes in this octree 
		@remark do not confuse this with the OctreeNode class which is derived from SceneNode
		@param num number of nodes in the octree
	*/
	void SetNumOctreeNodes(unsigned int num);

	/** Gets the given option for the scene traverser.
	    @remarks
		See setOption
	*/
	//bool getOption( const String &, void * );
	//bool getOptionKeys( StringVector &refKeys );
	void PullUpVisibility(GtpVisibility::HierarchyNode *node);
	/** Traverses given node.
		@param node current node
		@remark pushes children on distance queue
	*/
	void TraverseNode(GtpVisibility::HierarchyNode *node);
	void RenderNode(GtpVisibility::HierarchyNode *node);
	bool IsLeaf(GtpVisibility::HierarchyNode *node) const;
	bool HasGeometry(GtpVisibility::HierarchyNode *node) const;
	float GetSquaredDistance(GtpVisibility::HierarchyNode *node) const;
	
	void SetNodeVisible(GtpVisibility::HierarchyNode *node, const bool visible);
	void SetLastVisited(GtpVisibility::HierarchyNode *node, const unsigned int frameId);

	bool IsNodeVisible(GtpVisibility::HierarchyNode *node) const;
	unsigned int LastVisited(GtpVisibility::HierarchyNode *node) const;

	void VisualizeCulledNode(GtpVisibility::HierarchyNode *node, 
							 GtpVisibility::CullingType type);

	/*bool FindVisibleObjects(GtpVisibility::HierarchyNode *node,	
							InfoContainer<GtpVisibility::MeshInfo> *visibleGeometry, 
							bool includeChildren = false);*/
	void GetGeometry(GtpVisibility::HierarchyNode *node,
					 GtpVisibility::GeometryList *geometryList, 
					 bool includeChildren);

protected:
	/** Returns pointer to the bounding box of the node.
		@param node current hierarchy node
		@returns bounding box of current node
	*/
	AxisAlignedBox *GetBoundingBox(GtpVisibility::HierarchyNode *node);
	/** Returns squared distance of center of box with respect to the camera .
		@param cam current camera
		@param box axis aligned box
	*/
	Real GetSquaredViewDepth(const Camera* cam, const AxisAlignedBox* box) const;
	/** number of octree hierarchy nodes */
	unsigned int mNumOctreeNodes;
};
} // namespace Ogre

#endif // OctreeHierarchyInterface_H